@@ -1,9 +1,10 @@
 /*
-**  OSSP l2 - Logging Library
-**  Copyright (c) 2001-2002 The OSSP Project (http://www.ossp.org/)
-**  Copyright (c) 2001-2002 Cable & Wireless Deutschland (http://www.cw.com/de/)
+**  OSSP l2 - Flexible Logging
+**  Copyright (c) 2001-2005 Cable & Wireless <http://www.cw.com/>
+**  Copyright (c) 2001-2005 The OSSP Project <http://www.ossp.org/>
+**  Copyright (c) 2001-2005 Ralf S. Engelschall <rse@engelschall.com>
 **
-**  This file is part of OSSP L2, a flexible logging library which
+**  This file is part of OSSP l2, a flexible logging library which
 **  can be found at http://www.ossp.org/pkg/lib/l2/.
 **
 **  Permission to use, copy, modify, and distribute this software for
@@ -39,7 +40,7 @@
  * Especially any Apache and network specific kludges were removed again
  * and instead the formatting engine now can be extended by the caller
  * on-the-fly. It was then finally adjusted to be stand-alone for use
- * inside OSSP L2.
+ * inside OSSP.
  */
 
 /*
@@ -97,6 +98,7 @@
 #include <ctype.h>
 #include <math.h>
 
+#include "l2_config.h"
 #include "l2_ut_format.h"
 
 /* types which are locally use */
@@ -123,12 +125,13 @@ typedef unsigned long      u_quad_int;
 #ifndef TRUE
 #define TRUE (!FALSE)
 #endif
-#define S_NULL          "(NULL)"
+static char l2_util_fmt_null[] = "(NULL)";
+#define S_NULL          l2_util_fmt_null
 #define S_NULL_LEN      6
 #define FLOAT_DIGITS    6
 #define EXPONENT_LENGTH 10
 
-/* NUM_BUF_SIZE is the size of the buffer used for arithmetic 
+/* NUM_BUF_SIZE is the size of the buffer used for arithmetic
    conversions. This is a magic number; do NOT decrease it! */
 #define NUM_BUF_SIZE    512
 #define NDIG            80
@@ -148,7 +151,7 @@ typedef unsigned long      u_quad_int;
 #define l2_util_islower(c)  (islower(((unsigned char)(c))))
 #define l2_util_tolower(c)  (tolower((unsigned char)(c)))
 
-/* 
+/*
  * Convert decimal number to its string representation. The number of
  * digits is specified by ndigit decpt is set to the position of the
  * decimal point sign is set to 0 for positive, 1 for negative. buf must
@@ -161,12 +164,32 @@ typedef unsigned long      u_quad_int;
 #define l2_util_fcvt(arg,ndigits,decpt,sign,buf) \
         l2_util_cvt((arg), (ndigits), (decpt), (sign), 0, (buf))
 
+/* inlined modf(3) to avoid dependency to external libm on systems
+   (like Tru64, QNX, etc) where modf(3) is not part of libc.
+   We use trnc to avoid conflicts with global definitions of trunc and truncate */
+static double
+l2_util_modf(
+    double arg,
+    double *iptr)
+{
+    double fraction;
+    double integral;
+    long trnc;
+
+    trnc = (long)arg;
+    integral = (double)trnc;
+    fraction = arg - integral;
+    if (iptr != NULL)
+        *iptr = integral;
+    return fraction;
+}
+
 static char *
 l2_util_cvt(
-    double arg, 
-    int ndigits, 
-    int *decpt, 
-    int *sign, 
+    double arg,
+    int ndigits,
+    int *decpt,
+    int *sign,
     int eflag,
     char *buf)
 {
@@ -183,14 +206,14 @@ l2_util_cvt(
         *sign = TRUE;
         arg = -arg;
     }
-    arg = modf(arg, &fi);
+    arg = l2_util_modf(arg, &fi);
     p1 = &buf[NDIG];
 
     /* Do integer part */
     if (fi != 0) {
         p1 = &buf[NDIG];
         while (fi != 0 && p1 > &buf[0]) {
-            fj = modf(fi / 10, &fi);
+            fj = l2_util_modf(fi / 10, &fi);
             *--p1 = (int)((fj + .03) * 10) + '0';
             r2++;
         }
@@ -213,7 +236,7 @@ l2_util_cvt(
     }
     while (p <= p1 && p < &buf[NDIG]) {
         arg *= 10;
-        arg = modf(arg, &fj);
+        arg = l2_util_modf(arg, &fj);
         *p++ = (int) fj + '0';
     }
     if (p1 >= &buf[NDIG]) {
@@ -242,9 +265,9 @@ l2_util_cvt(
 
 static char *
 l2_util_gcvt(
-    double number, 
-    int ndigit, 
-    char *buf, 
+    double number,
+    int ndigit,
+    char *buf,
     int altform)
 {
     int sign;
@@ -326,7 +349,7 @@ l2_util_gcvt(
 }
 
 /*
- * Convert a string to decimal value 
+ * Convert a string to decimal value
  */
 #define NUM(c) ((c) - '0')
 #define STR_TO_DEC(str, num) {    \
@@ -336,7 +359,7 @@ l2_util_gcvt(
         num += NUM(*str++) ;      \
     }                             \
 }
-     
+
 /*
  * This macro does zero padding so that the precision requirement is
  * satisfied. The padding is done by adding '0's to the left of the
@@ -351,13 +374,13 @@ l2_util_gcvt(
     }
 
 /*
- * This macro does padding. 
+ * This macro does padding.
  * The padding is done by printing the character ch.
  */
 #define PAD(width, len, ch) \
     do {                            \
         INS_CHAR(ch, sp, bep, cc);  \
-        width-- ;                   \
+        width--;                    \
     } while (width > len)
 
 /*
@@ -385,9 +408,9 @@ l2_util_gcvt(
  */
 static char *
 conv_10(
-    register long_int num, 
+    register long_int num,
     register int is_unsigned,
-    register int *is_negative, 
+    register int *is_negative,
     char *buf_end,
     register size_t *len)
 {
@@ -400,7 +423,7 @@ conv_10(
     }
     else {
         *is_negative = (num < 0);
-        /* On a 2's complement machine, negating the most negative integer 
+        /* On a 2's complement machine, negating the most negative integer
            results in a number that cannot be represented as a signed integer.
            Here is what we do to obtain the number's magnitude:
                 a. add 1 to the number
@@ -427,9 +450,9 @@ conv_10(
 
 static char *
 conv_10_quad(
-    quad_int num, 
+    quad_int num,
     register int is_unsigned,
-    register int *is_negative, 
+    register int *is_negative,
     char *buf_end,
     register size_t *len)
 {
@@ -442,7 +465,7 @@ conv_10_quad(
     }
     else {
         *is_negative = (num < 0);
-        /* On a 2's complement machine, negating the most negative integer 
+        /* On a 2's complement machine, negating the most negative integer
            result in a number that cannot be represented as a signed integer.
            Here is what we do to obtain the number's magnitude:
                 a. add 1 to the number
@@ -474,12 +497,12 @@ conv_10_quad(
  */
 static char *
 conv_fp(
-    register char format, 
+    register char format,
     register double num,
-    int add_dp, 
-    int precision, 
+    int add_dp,
+    int precision,
     int *is_negative,
-    char *buf, 
+    char *buf,
     size_t *len)
 {
     register char *s = buf;
@@ -535,7 +558,7 @@ conv_fp(
         *s++ = format;          /* either e or E */
         decimal_point--;
         if (decimal_point != 0) {
-            p = conv_10((long_int) decimal_point, FALSE, &exponent_is_negative, 
+            p = conv_10((long_int) decimal_point, FALSE, &exponent_is_negative,
                         &temp[EXPONENT_LENGTH], &t_len);
             *s++ = exponent_is_negative ? '-' : '+';
             /* Make sure the exponent has at least 2 digits */
@@ -572,10 +595,10 @@ static const char upper_digits[] = "0123456789ABCDEF";
 
 static char *
 conv_p2(
-    register u_long_int num, 
+    register u_long_int num,
     register int nbits,
-    char format, 
-    char *buf_end, 
+    char format,
+    char *buf_end,
     register size_t *len)
 {
     register int mask = (1 << nbits) - 1;
@@ -592,10 +615,10 @@ conv_p2(
 
 static char *
 conv_p2_quad(
-    u_quad_int num, 
+    u_quad_int num,
     register int nbits,
-    char format, 
-    char *buf_end, 
+    char format,
+    char *buf_end,
     register size_t *len)
 {
     register int mask = (1 << nbits) - 1;
@@ -610,20 +633,20 @@ conv_p2_quad(
     return p;
 }
 
-/* 
+/*
  * l2_util_format(), the generic printf-style formatting routine
  * and heart of this piece of source.
  */
-int 
+int
 l2_util_format(
-    l2_util_format_t *vbuff, 
-    const char *fmt, 
+    l2_util_format_t *vbuff,
+    const char *fmt,
     va_list ap)
 {
     register char *sp;
     register char *bep;
     register int cc = 0;
-    register int i;
+    register unsigned int i;
 
     char *s = NULL;
     char *q;
@@ -911,7 +934,7 @@ l2_util_format(
                     else {
                         /* use &num_buf[1], so that we have room for the sign */
                         s = conv_fp(*fmt, fp_num, alternate_form,
-                                    (adjust_precision == FALSE) ? FLOAT_DIGITS : precision, 
+                                    (adjust_precision == FALSE) ? FLOAT_DIGITS : precision,
                                     &is_negative, &num_buf[1], &s_len);
                         if (is_negative)
                             prefix_char = '-';
@@ -986,7 +1009,7 @@ l2_util_format(
                     break;
 
                 /*
-                 * Pointer argument type. 
+                 * Pointer argument type.
                  */
                 case 'p':
 #if defined(SIZEOF_LONG_LONG) && (SIZEOF_LONG_LONG == SIZEOF_VOID_P)
@@ -1020,8 +1043,8 @@ l2_util_format(
                     s = NULL;
                     if (vbuff->format != NULL) {
                         vbuff->format(vbuff,
-                                      &prefix_char, &pad_char, &s, &s_len, 
-                                      num_buf, NUM_BUF_SIZE, extinfo, *fmt, &ap);
+                                      &prefix_char, &pad_char, &s, &s_len,
+                                      num_buf, NUM_BUF_SIZE, extinfo, *fmt, (va_list *)(void *)&ap);
                         if (s == NULL)
                             return -1;
                     }
@@ -1051,7 +1074,7 @@ l2_util_format(
             }
 
             /*
-             * Print the string s. 
+             * Print the string s.
              */
             for (i = s_len; i != 0; i--) {
                 INS_CHAR(*s, sp, bep, cc);
@@ -1131,12 +1154,12 @@ char *l2_util_vasprintf(const char *fmt, va_list ap)
     char *s;
     int rv;
 
-    apbak = ap;
+    va_copy(apbak, ap);
     if ((rv = l2_util_vsprintf(NULL, -1, fmt, ap)) == -1)
         return NULL;
     if ((s = malloc(rv+1)) == NULL)
         return NULL;
-    ap = apbak;
+    va_copy(ap, apbak);
     if ((rv = l2_util_vsprintf(s, rv+1, fmt, ap)) == -1) {
         free(s);
         return NULL;
